@@ -27,8 +27,12 @@
 #include <libgda/gda-data-model-dir.h>
 #include <libgda/gda-data-model-extra.h>
 
-#ifdef HAVE_GNOMEVFS
-#include <libgnomevfs/gnome-vfs-mime.h>
+#ifdef HAVE_GIO
+#include <gio/gio.h>
+#else
+  #ifdef HAVE_GNOMEVFS
+  #include <libgnomevfs/gnome-vfs-mime.h>
+  #endif
 #endif
 
 /* Use the RSA reference implementation included in the RFC-1321, http://www.freesoft.org/CIE/RFC/1321/ */
@@ -50,12 +54,14 @@
 #include "dir-blob-op.h"
 
 struct _GdaDataModelDirPrivate {
-	gchar  *basedir;
-	GSList *errors; /* list of errors as GError structures */
-	GSList *columns; /* list of GdaColumn objects */
+	gchar     *basedir;
+	GSList    *errors; /* list of errors as GError structures */
+	GSList    *columns; /* list of GdaColumn objects */
 
 	GPtrArray *rows; /* array of FileRow pointers */
 	gint       upd_row; /* internal usage when updating contents */
+
+	GValue    *tmp_value; /* GValue returned by gda_data_model_get_value_at() */
 };
 
 /* Row implementation details */
@@ -117,12 +123,12 @@ static void gda_data_model_dir_get_property (GObject *object,
 					     GParamSpec *pspec);
 
 /* GdaDataModel interface */
-static void                 gda_data_model_dir_data_model_init (GdaDataModelClass *iface);
+static void                 gda_data_model_dir_data_model_init (GdaDataModelIface *iface);
 static gint                 gda_data_model_dir_get_n_rows      (GdaDataModel *model);
 static gint                 gda_data_model_dir_get_n_columns   (GdaDataModel *model);
 static GdaColumn           *gda_data_model_dir_describe_column (GdaDataModel *model, gint col);
 static GdaDataModelAccessFlags gda_data_model_dir_get_access_flags(GdaDataModel *model);
-static const GValue      *gda_data_model_dir_get_value_at    (GdaDataModel *model, gint col, gint row);
+static const GValue        *gda_data_model_dir_get_value_at    (GdaDataModel *model, gint col, gint row, GError **error);
 static GdaValueAttribute    gda_data_model_dir_get_attributes_at (GdaDataModel *model, gint col, gint row);
 
 static gboolean             gda_data_model_dir_set_value_at (GdaDataModel *model, gint col, gint row, const GValue *value, GError **error);
@@ -140,7 +146,7 @@ static GObjectClass *parent_class = NULL;
  * Object init and dispose
  */
 static void
-gda_data_model_dir_data_model_init (GdaDataModelClass *iface)
+gda_data_model_dir_data_model_init (GdaDataModelIface *iface)
 {
         iface->i_get_n_rows = gda_data_model_dir_get_n_rows;
         iface->i_get_n_columns = gda_data_model_dir_get_n_columns;
@@ -155,6 +161,7 @@ gda_data_model_dir_data_model_init (GdaDataModelClass *iface)
         iface->i_iter_prev = NULL;
 
         iface->i_set_value_at = gda_data_model_dir_set_value_at;
+	iface->i_iter_set_value = NULL;
         iface->i_set_values = gda_data_model_dir_set_values;
         iface->i_append_values = gda_data_model_dir_append_values;
         iface->i_append_row = NULL;
@@ -176,6 +183,7 @@ gda_data_model_dir_init (GdaDataModelDir *model,
 	model->priv->basedir = NULL;
 	model->priv->columns = NULL;
 	model->priv->rows = g_ptr_array_new (); /* array of FileRow pointers */
+	model->priv->tmp_value = NULL;
 }
 
 static void
@@ -189,7 +197,7 @@ gda_data_model_dir_class_init (GdaDataModelDirClass *klass)
         object_class->set_property = gda_data_model_dir_set_property;
         object_class->get_property = gda_data_model_dir_get_property;
         g_object_class_install_property (object_class, PROP_BASEDIR,
-                                         g_param_spec_string ("basedir", "Base directory", NULL, NULL,
+                                         g_param_spec_string ("basedir", NULL, "Base directory", NULL,
                                                               G_PARAM_READABLE | G_PARAM_WRITABLE |
                                                               G_PARAM_CONSTRUCT_ONLY));
 
@@ -211,6 +219,11 @@ gda_data_model_dir_dispose (GObject * object)
 	g_return_if_fail (GDA_IS_DATA_MODEL_DIR (model));
 
 	if (model->priv) {
+		if (model->priv->tmp_value) {
+			gda_value_free (model->priv->tmp_value);
+			model->priv->tmp_value = NULL;
+		}
+
 		if (model->priv->basedir) {
 			g_free (model->priv->basedir);
 			model->priv->basedir = NULL;
@@ -241,7 +254,7 @@ add_error (GdaDataModelDir *model, const gchar *err)
 {
 	GError *error = NULL;
 
-        g_set_error (&error, 0, 0, err);
+        g_set_error (&error, 0, 0, "%s", err);
         model->priv->errors = g_slist_append (model->priv->errors, error);
 }
 
@@ -255,6 +268,7 @@ gda_data_model_dir_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaDataModelDirClass),
 			(GBaseInitFunc) NULL,
@@ -272,8 +286,12 @@ gda_data_model_dir_get_type (void)
                         NULL
                 };
 
-		type = g_type_register_static (G_TYPE_OBJECT, "GdaDataModelDir", &info, 0);
-		g_type_add_interface_static (type, GDA_TYPE_DATA_MODEL, &data_model_info);
+		g_static_mutex_lock (&registering);
+		if (type == 0) {
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaDataModelDir", &info, 0);
+			g_type_add_interface_static (type, GDA_TYPE_DATA_MODEL, &data_model_info);
+		}
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -286,7 +304,6 @@ gda_data_model_dir_set_property (GObject *object,
 {
         GdaDataModelDir *model;
         const gchar *string;
-	static gboolean basedir_set = FALSE;
 
         model = GDA_DATA_MODEL_DIR (object);
         if (model->priv) {
@@ -297,14 +314,12 @@ gda_data_model_dir_set_property (GObject *object,
 				model->priv->basedir = NULL;
 			}
 			string = g_value_get_string (value);
-			if (string) {
+			if (string) 
 				model->priv->basedir = g_strdup (string);
-				basedir_set = TRUE;
-			}
 			break;
 		}
 
-		if (basedir_set) {
+		if (model->priv->basedir) {
 			/* create columns */
 			model->priv->columns = NULL;
 			GdaColumn *column;
@@ -313,42 +328,42 @@ gda_data_model_dir_set_property (GObject *object,
 			column = gda_column_new ();
 			model->priv->columns = g_slist_append (model->priv->columns , column);
 			gda_column_set_name (column, "dir_name");
-			gda_column_set_title (column, "dir_name");
+			gda_column_set_description (column, "dir_name");
 			gda_column_set_g_type (column, G_TYPE_STRING);
 
 			/* COL_FILENAME */
 			column = gda_column_new ();
 			model->priv->columns = g_slist_append (model->priv->columns , column);
 			gda_column_set_name (column, "file_name");
-			gda_column_set_title (column, "file_name");
+			gda_column_set_description (column, "file_name");
 			gda_column_set_g_type (column, G_TYPE_STRING);
 
 			/* COL_SIZE */
 			column = gda_column_new ();
 			model->priv->columns = g_slist_append (model->priv->columns , column);
 			gda_column_set_name (column, "size");
-			gda_column_set_title (column, "size");
+			gda_column_set_description (column, "size");
 			gda_column_set_g_type (column, G_TYPE_UINT);
 
 			/* COL_MIME */
 			column = gda_column_new ();
 			model->priv->columns = g_slist_append (model->priv->columns , column);
 			gda_column_set_name (column, "mime_type");
-			gda_column_set_title (column, "mime_type");
+			gda_column_set_description (column, "mime_type");
 			gda_column_set_g_type (column, G_TYPE_STRING);
 
 			/* COL_MD5SUM */
 			column = gda_column_new ();
 			model->priv->columns = g_slist_append (model->priv->columns , column);
 			gda_column_set_name (column, "md5sum");
-			gda_column_set_title (column, "md5sum");
+			gda_column_set_description (column, "md5sum");
 			gda_column_set_g_type (column, G_TYPE_STRING);
 
 			/* COL_DATA */
 			column = gda_column_new ();
 			model->priv->columns = g_slist_append (model->priv->columns , column);
 			gda_column_set_name (column, "data");
-			gda_column_set_title (column, "data");
+			gda_column_set_description (column, "data");
 			gda_column_set_g_type (column, GDA_TYPE_BLOB);
 
 			/* number of rows */
@@ -539,7 +554,7 @@ update_file_md5sum (FileRow *row, const gchar *complete_filename)
  md5end:
 	if (value) {
 		if (row->md5sum_value && (G_VALUE_TYPE (row->md5sum_value) == G_TYPE_STRING)
-		    && !gda_value_compare_ext (row->md5sum_value, value))
+		    && !gda_value_compare (row->md5sum_value, value))
 			changed = FALSE;
 		else {
 			if (row->md5sum_value)
@@ -566,8 +581,23 @@ update_file_mime (FileRow *row, const gchar *complete_filename)
 {
 	gboolean changed = TRUE;
 	GValue *value = NULL;
-
-#ifdef HAVE_GNOMEVFS
+#ifdef HAVE_GIO
+	GFile *file;
+	GFileInfo *info;
+	file = g_file_new_for_path (complete_filename);
+	info = g_file_query_info (file,
+				  G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE,
+				  G_FILE_QUERY_INFO_NONE, NULL, NULL);
+	if (info) {
+		value = gda_value_new (G_TYPE_STRING);
+                g_value_set_string (value, g_file_info_get_content_type (info));
+		g_object_unref (info);
+	}
+	else
+		value = gda_value_new_null ();
+	g_object_unref (file);
+#else
+  #ifdef HAVE_GNOMEVFS
 	const gchar *mime;
 	
 	mime = gnome_vfs_get_file_mime_type (complete_filename, NULL, FALSE);
@@ -577,13 +607,14 @@ update_file_mime (FileRow *row, const gchar *complete_filename)
 	}
 	else 
 		value = gda_value_new_null ();
-#else
+  #else
 	value = gda_value_new_null ();
+  #endif
 #endif
 
 	if (value) {
 		if (row->mime_value && (G_VALUE_TYPE (row->mime_value) == G_TYPE_STRING)
-		    && !gda_value_compare_ext (row->mime_value, value))
+		    && !gda_value_compare (row->mime_value, value))
 			changed = FALSE;
 		else {
 			if (row->mime_value)
@@ -746,27 +777,33 @@ gda_data_model_dir_get_access_flags (GdaDataModel *model)
 }
 
 static const GValue *
-gda_data_model_dir_get_value_at (GdaDataModel *model, gint col, gint row)
+gda_data_model_dir_get_value_at (GdaDataModel *model, gint col, gint row, GError **error)
 {
 	GdaDataModelDir *imodel;
 	GValue *value = NULL;
 	FileRow *frow;
-	static GValue *tmp_value = NULL;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_DIR (model), NULL);
 	imodel = GDA_DATA_MODEL_DIR (model);
 	g_return_val_if_fail (imodel->priv, NULL);
 
 	if ((col < 0) || (col > COL_LAST)) {
-		add_error (imodel, _("Column number out of range"));
+		gchar *tmp;
+		tmp = g_strdup_printf (_("Column %d out of range (0-%d)"), col, COL_LAST-1);
+		add_error (imodel, tmp);
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_COLUMN_OUT_OF_RANGE_ERROR,
+			      "%s", tmp);
+		g_free (tmp);
 		return NULL;
 	}
 
 	if (row >= imodel->priv->rows->len) {
 		gchar *str;
-                str = g_strdup_printf (_("Row %d out of range 0 - %d"), row,
+                str = g_strdup_printf (_("Row %d out of range (0-%d)"), row,
 				       imodel->priv->rows->len - 1);
 		add_error (imodel, str);
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_OUT_OF_RANGE_ERROR,
+			      "%s", str);
 		g_free (str);
                 return NULL;
         }
@@ -777,10 +814,10 @@ gda_data_model_dir_get_value_at (GdaDataModel *model, gint col, gint row)
 		case COL_DIRNAME: {
 			gchar *tmp;
 			tmp = compute_dirname (imodel, frow);
-			if (!tmp_value)
-				tmp_value = gda_value_new (G_TYPE_STRING);
-			g_value_take_string (tmp_value, tmp);
-			value = tmp_value;
+			if (!imodel->priv->tmp_value)
+				imodel->priv->tmp_value = gda_value_new (G_TYPE_STRING);
+			g_value_take_string (imodel->priv->tmp_value, tmp);
+			value = imodel->priv->tmp_value;
 			break;
 		}
 		case COL_FILENAME:
@@ -830,6 +867,9 @@ gda_data_model_dir_get_value_at (GdaDataModel *model, gint col, gint row)
 			break;
 		}
 	}
+	else
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR,
+			      "%s", _("Row not found"));
 
 	return value;
 }
@@ -844,7 +884,10 @@ gda_data_model_dir_get_attributes_at (GdaDataModel *model, gint col, gint row)
 	g_return_val_if_fail (imodel->priv, 0);
 
 	if ((col < 0) || (col > COL_LAST)) {
-		add_error (imodel, _("Column number out of range"));
+		gchar *tmp;
+		tmp = g_strdup_printf (_("Column %d out of range (0-%d)"), col, COL_LAST-1);
+		add_error (imodel, tmp);
+		g_free (tmp);
 		return 0;
 	}
 
@@ -883,9 +926,11 @@ gda_data_model_dir_set_value_at (GdaDataModel *model, gint col, gint row, const
 	g_return_val_if_fail (imodel->priv, FALSE);
 
 	if ((col < 0) || (col > COL_LAST)) {
-		add_error (imodel, _("Column number out of range"));
-		g_set_error (error, 0, 0,
-			     _("Column number out of range"));
+		gchar *tmp;
+		tmp = g_strdup_printf (_("Column %d out of range (0-%d)"), col, COL_LAST-1);
+		add_error (imodel, tmp);
+		g_set_error (error, 0, 0, "%s", tmp);
+		g_free (tmp);
 		return FALSE;
 	}
 
@@ -923,10 +968,10 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 
 	if (row >= imodel->priv->rows->len) {
 		gchar *str;
-                str = g_strdup_printf (_("Row %d out of range 0 - %d"), row,
+                str = g_strdup_printf (_("Row %d out of range (0-%d)"), row,
 				       imodel->priv->rows->len - 1);
 		add_error (imodel, str);
-		g_set_error (error, 0, 0, str);
+		g_set_error (error, 0, 0, "%s", str);
 		g_free (str);
                 return FALSE;
         }
@@ -935,7 +980,10 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 
 	for (col = 0, list = values; list; list = list->next, col++) {
 		GValue *value = (GValue *) list->data;
-		if (!value || !gda_value_compare_ext (value, gda_data_model_get_value_at (model, col, row))) 
+		const GValue *cvalue = gda_data_model_get_value_at (model, col, row, error);
+		if (!cvalue)
+			return FALSE;
+		if (!value || !gda_value_compare (value, cvalue)) 
 			continue;
 
 		switch (col) {
@@ -944,7 +992,7 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 		case COL_MD5SUM:
 		default:
 			add_error (imodel, _("Column cannot be modified"));
-			g_set_error (error, 0, 0,
+			g_set_error (error, 0, 0, "%s", 
 				     _("Column cannot be modified"));
 			return FALSE;
 		case COL_DIRNAME: {
@@ -959,7 +1007,7 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 			if ((len < base_len) ||
 			    (strncmp (new_path, imodel->priv->basedir, base_len))) {
 				add_error (imodel, _("New path must be a subpath of the base directory"));
-				g_set_error (error, 0, 0,
+				g_set_error (error, 0, 0, "%s", 
 					     _("New path must be a subpath of the base directory"));
 				return FALSE;
 			}
@@ -1003,7 +1051,7 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 					str = g_strdup_printf (_("Could not rename file '%s' to '%s'"), 
 							       filename, new_filename);
 					add_error (imodel, str);
-					g_set_error (error, 0, 0, str);
+					g_set_error (error, 0, 0, "%s", str);
 					g_free (str);
 					g_free (new_filename);
 					g_free (filename);
@@ -1026,7 +1074,7 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 				gchar *str;
 				str = g_strdup_printf (_("Could not create directory '%s'"), new_path);
 				add_error (imodel, str);
-				g_set_error (error, 0, 0, str);
+				g_set_error (error, 0, 0, "%s", str);
 				g_free (str);
 				g_free (old_path);
 				return FALSE;
@@ -1046,7 +1094,7 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 				gchar *str;
 				str = g_strdup_printf (_("Could not rename file '%s' to '%s'"), filename, new_filename);
 				add_error (imodel, str);
-				g_set_error (error, 0, 0, str);
+				g_set_error (error, 0, 0, "%s", str);
 				g_free (str);
 				g_free (new_filename);
 				g_free (filename);
@@ -1095,7 +1143,7 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 					gchar *str;
 					str = g_strdup_printf (_("Could not overwrite contents of file '%s'"), filename);
 					add_error (imodel, str);
-					g_set_error (error, 0, 0, str);
+					g_set_error (error, 0, 0, "%s", str);
 					g_free (str);
 					g_object_unref (op);
 					g_free (filename);
@@ -1112,7 +1160,7 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 			}
 			else {
 				add_error (imodel, _("Wrong type of data"));
-				g_set_error (error, 0, 0, _("Wrong type of data"));
+				g_set_error (error, 0, 0, "%s", _("Wrong type of data"));
 				return FALSE;
 			}
 			break;
@@ -1154,7 +1202,7 @@ gda_data_model_dir_append_values (GdaDataModel *model, const GList *values, GErr
 		case COL_MD5SUM:
 		default:
 			add_error (imodel, _("Column cannot be set"));
-			g_set_error (error, 0, 0,
+			g_set_error (error, 0, 0, "%s", 
 				     _("Column cannot be set"));
 			return -1;
 		case COL_DIRNAME:
@@ -1166,7 +1214,7 @@ gda_data_model_dir_append_values (GdaDataModel *model, const GList *values, GErr
 				if ((len < base_len) ||
 				    (strncmp (dirname, imodel->priv->basedir, base_len))) {
 					add_error (imodel, _("New path must be a subpath of the base directory"));
-					g_set_error (error, 0, 0,
+					g_set_error (error, 0, 0, "%s", 
 						     _("New path must be a subpath of the base directory"));
 					return -1;
 				}
@@ -1228,7 +1276,7 @@ gda_data_model_dir_append_values (GdaDataModel *model, const GList *values, GErr
 				gchar *str;
 				str = g_strdup_printf (_("Cannot set contents of filename '%s'"), complete_filename);
 				add_error (imodel, str);
-				g_set_error (error, 0, 0, str);
+				g_set_error (error, 0, 0, "%s", str);
 				g_free (str);
 				if (bin_to_free)
 					g_free (bin_data);
@@ -1239,14 +1287,14 @@ gda_data_model_dir_append_values (GdaDataModel *model, const GList *values, GErr
 			gchar *str;
 			str = g_strdup_printf (_("Cannot create directory '%s'"), dirname);
 			add_error (imodel, str);
-			g_set_error (error, 0, 0, str);
+			g_set_error (error, 0, 0, "%s", str);
 			g_free (str);
 			return -1;
 		}
 	}
 	else {
 		add_error (imodel, _("Cannot add row: filename missing"));
-		g_set_error (error, 0, 0,
+		g_set_error (error, 0, 0, "%s", 
 			     _("Cannot add row: filename missing"));
 		return -1;
 	}
@@ -1267,10 +1315,10 @@ gda_data_model_dir_remove_row (GdaDataModel *model, gint row, GError **error)
 
 	if (row >= imodel->priv->rows->len) {
 		gchar *str;
-                str = g_strdup_printf (_("Row %d out of range 0 - %d"), row,
+                str = g_strdup_printf (_("Row %d out of range (0-%d)"), row,
 				       imodel->priv->rows->len - 1);
 		add_error (imodel, str);
-		g_set_error (error, 0, 0, str);
+		g_set_error (error, 0, 0, "%s", str);
 		g_free (str);
                 return FALSE;
         }
@@ -1286,7 +1334,7 @@ gda_data_model_dir_remove_row (GdaDataModel *model, gint row, GError **error)
 		gchar *str;
 		str = g_strdup_printf (_("Cannot remove file '%s'"), filename);
 		add_error (imodel, str);
-		g_set_error (error, 0, 0, str);
+		g_set_error (error, 0, 0, "%s", str);
 		g_free (str);
 		g_free (filename);
 		return FALSE;
