@@ -217,16 +217,20 @@ set_from_string (GValue *value, const gchar *as_string)
 		}
 	}
 	else if (type == GDA_TYPE_BINARY) {
-		GdaBinary binary;
-		retval = gda_string_to_binary (as_string, &binary);
-		if (retval)
-			gda_value_set_binary (value, &binary);
+		GdaBinary *bin;
+		bin = gda_string_to_binary (as_string);
+		if (bin) {
+			gda_value_take_binary (value, bin);
+			retval = TRUE;
+		}
 	}
 	else if (type == GDA_TYPE_BLOB) {
-		GdaBlob blob;
-		retval = gda_string_to_blob (as_string, &blob);
-		if (retval)
-			gda_value_set_blob (value, &blob);
+		GdaBlob *blob;
+		blob = gda_string_to_blob (as_string);
+		if (blob) {
+			gda_value_take_blob (value, blob);
+			retval = TRUE;
+		}
 	}
 
 
@@ -256,7 +260,7 @@ static void
 string_to_binary (const GValue *src, GValue *dest) 
 {
 	/* FIXME: add more checks*/
-	GdaBinary *binary;
+	GdaBinary *bin;
 	const gchar *as_string;
 	
 	g_return_if_fail (G_VALUE_HOLDS_STRING (src) &&
@@ -264,9 +268,9 @@ string_to_binary (const GValue *src, GValue *dest)
 	
 	as_string = g_value_get_string (src);
 	
-	binary = g_new0 (GdaBinary, 1);
-	gda_string_to_binary (as_string, binary);
-	gda_value_take_binary (dest, binary);
+	bin = gda_string_to_binary (as_string);
+	g_return_if_fail (bin);
+	gda_value_take_binary (dest, bin);
 }
 
 static void 
@@ -361,8 +365,8 @@ string_to_blob (const GValue *src, GValue *dest)
 	
 	as_string = g_value_get_string (src);
 	
-	blob = g_new0 (GdaBlob, 1);
-	gda_string_to_blob (as_string, blob);
+	blob = gda_string_to_blob (as_string);
+	g_return_if_fail (blob);
 	gda_value_take_blob (dest, blob);
 }
 
@@ -759,7 +763,7 @@ gda_numeric_copy (gpointer boxed)
 
 /**
  * gda_numeric_free
- * @boxed:
+ * @boxed: a #GdaNumeric pointer
  *
  * Deallocates all memory associated to the given @boxed
  */
@@ -1181,8 +1185,8 @@ gda_value_new_timestamp_from_timet (time_t val)
         ltm = localtime ((const time_t *) &val);
         if (ltm) {
                 GdaTimestamp tstamp;
-                tstamp.year = ltm->tm_year;
-                tstamp.month = ltm->tm_mon;
+                tstamp.year = ltm->tm_year + 1900;
+                tstamp.month = ltm->tm_mon + 1;
                 tstamp.day = ltm->tm_mday;
                 tstamp.hour = ltm->tm_hour;
                 tstamp.minute = ltm->tm_min;
@@ -1418,7 +1422,7 @@ gda_value_set_binary (GValue *value, const GdaBinary *binary)
  * argument is not copied, but used as-is and it should be considered owned by @value.
  */
 void
-gda_value_take_binary (GValue *value, const GdaBinary *binary)
+gda_value_take_binary (GValue *value, GdaBinary *binary)
 {
 	g_return_if_fail (value);
 	g_return_if_fail (binary);
@@ -1474,7 +1478,7 @@ gda_value_get_blob (const GValue *value)
  * argument is not copied, but used as-is and it should be considered owned by @value.
  */
 void
-gda_value_take_blob (GValue *value, const GdaBlob *blob)
+gda_value_take_blob (GValue *value, GdaBlob *blob)
 {
 	g_return_if_fail (value);
 	g_return_if_fail (blob);
@@ -2481,8 +2485,8 @@ gda_ushort_get_type (void) {
  * @bin: a correctly filled @GdaBinary structure
  * @maxlen: a maximum len used to truncate, or 0 for no maximum length
  *
- * Converts all the non printable characters of bin->data into the \xxx representation
- * where xxx is the octal representation of the byte, and the '\' (backslash) character
+ * Converts all the non printable characters of bin->data into the "\xyz" representation
+ * where "xyz" is the octal representation of the byte, and the '\' (backslash) character
  * is converted to "\\".
  *
  * Returns: a new string from @bin
@@ -2491,69 +2495,62 @@ gchar *
 gda_binary_to_string (const GdaBinary *bin, guint maxlen)
 {
 	gint nb_rewrites = 0;
-	gchar *ptr, *hold;
+	gchar *sptr, *rptr;
 	gulong realsize = MYMIN (bin->binary_length, maxlen);
 
 	gchar *retval;
 	glong offset = 0;
-	gunichar unichar;
 
 	if (!bin->data || (bin->binary_length == 0))
 		return g_strdup ("");
 
 	/* compute number of char rewrites */
-	ptr = (gchar*)bin->data;
-	while (offset < realsize) {
-		unichar = g_utf8_get_char_validated (ptr, -1);
-		if ((*ptr == '\n') ||
-		    ((*ptr != '\\') && (unichar > 0) && g_unichar_isprint (unichar))) {
-			hold = ptr;
-			ptr = g_utf8_next_char (ptr);
-			offset += ptr - hold;
-		}
-		else {
-			ptr++;
+	for (offset = 0, sptr = (gchar*) bin->data; 
+	     offset < realsize; 
+	     offset ++, sptr++) {
+		if ((*sptr != '\n') && ((*sptr == '\\') || !g_ascii_isprint (*sptr)))
 			nb_rewrites++;
-			offset ++;
-		}
 	}
 	
 	/* mem allocation and copy */
 	retval = g_malloc0 (realsize + nb_rewrites * 4 + 1);
-	memcpy (retval, bin->data, realsize);
-	ptr = retval;
+	rptr = retval;
+	sptr = (gchar*) bin->data;
 	offset = 0;
 	while (offset < realsize) {
-		unichar = g_utf8_get_char_validated (ptr, -1);
 		/* g_print (">%x<\n", (guchar) *ptr); */
-		if ((*ptr == '\n') ||
-		    ((*ptr != '\\') && (unichar > 0) && g_unichar_isprint (unichar))) {
-			hold = ptr;
-			ptr = g_utf8_next_char (ptr);
-			offset += ptr - hold;
+		if ((*sptr == '\n') || ((*sptr != '\\') && g_ascii_isprint (*sptr))) {
+			*rptr = *sptr;
+			rptr ++;
 		}
 		else {
-			if (*ptr == '\\') {
-				g_memmove (ptr+2, ptr+1, realsize - offset);
-
-				*(ptr+1) = '\\';
-				ptr += 2;
-				offset ++;
+			if (*sptr == '\\') {
+				*rptr = '\\';
+				rptr++;
+				*rptr = '\\';
+				rptr++;
 			}
 			else {
-				guchar val = *ptr;
-				g_memmove (ptr+4, ptr+1, realsize - offset);
-				*ptr = '\\';
-				*(ptr+1) = val / 49 + '0';
-				val = val % 49;
-				*(ptr+2) = val / 7 + '0';
-				val = val % 7;
-				*(ptr+3) = val + '0';
-				
-				ptr += 4;
-				offset ++;
+				guchar val = *sptr;
+
+				*rptr = '\\';
+				rptr++;
+
+				*rptr = val / 64 + '0';
+				rptr++;
+				val = val % 64;
+
+				*rptr = val / 8 + '0';
+				val = val % 8;
+				rptr++;
+
+				*rptr = val + '0';
+				rptr++;
 			}
 		}
+
+		sptr++;
+		offset ++;
 	}
 
 	return retval;
@@ -2562,62 +2559,69 @@ gda_binary_to_string (const GdaBinary *bin, guint maxlen)
 /**
  * gda_string_to_binary
  * @str: a string to convert
- * @bin: a non filled @GdaBinary structure
  *
- * Performs the reverse of gda_binary_to_string().
+ * Performs the reverse of gda_binary_to_string() (note that for any "\xyz" succession
+ * of 4 characters where "xyz" represents a valid octal value, the resulting read value will
+ * be modulo 256)
  *
- * Returns: TRUE if no error were found in @str, or FALSE otherwise
+ * Returns: a new #GdaBinary if no error were found in @str, or NULL otherwise
  */
-gboolean
-gda_string_to_binary (const gchar *str, GdaBinary *bin)
+GdaBinary *
+gda_string_to_binary (const gchar *str)
 {
+	GdaBinary *bin;
 	glong len = 0, total;
-	gchar *ptr;
+	gchar *rptr;
+	const gchar *sptr;
 	gchar *retval;
-	glong offset = 0;
 
 	if (!str) {
+		bin = g_new0 (GdaBinary, 1);
 		bin->data = NULL;
 		bin->binary_length = 0;
-		return TRUE;
+		return bin;
 	}
 
 	total = strlen (str);
-	retval = g_memdup (str, total);
-	ptr = (gchar *) retval;
-	while (offset < total) {
-		if (*ptr == '\\') {
-			if (*(ptr+1) == '\\') {
-				g_memmove (ptr+1, ptr+2, total - offset);
-				offset += 2;
+	retval = g_new0 (gchar, total + 1);
+	sptr = str;
+	rptr = (gchar *) retval;
+
+	while (*sptr) {
+		if (*sptr == '\\') {
+			if (*(sptr+1) == '\\') {
+				*rptr = '\\';
+				sptr += 2;
 			}
 			else {
-				if ((*(ptr+1) >= '0') && (*(ptr+1) <= '9') &&
-				    (*(ptr+2) >= '0') && (*(ptr+2) <= '9') &&
-				    (*(ptr+3) >= '0') && (*(ptr+3) <= '9')) {
-					*ptr = (*(ptr+1) - '0') * 49 +
-						(*(ptr+2) - '0') * 7 +
-						*(ptr+3) - '0';
-					g_memmove (ptr+1, ptr+4, total - offset);
-					offset += 4;
+				if ((*(sptr+1) >= '0') && (*(sptr+1) <= '7') &&
+				    (*(sptr+2) >= '0') && (*(sptr+2) <= '7') &&
+				    (*(sptr+3) >= '0') && (*(sptr+3) <= '7')) {
+					*rptr = (*(sptr+1) - '0') * 64 +
+						(*(sptr+2) - '0') * 8 +
+						(*(sptr+3) - '0');
+					sptr += 4;
 				}
 				else {
 					g_free (retval);
-					return FALSE;
+					return NULL;
 				}
 			}
 		}
-		else
-			offset ++;
+		else {
+			*rptr = *sptr;
+			sptr++;
+		}
 
-		ptr++;
+		rptr++;
 		len ++;
 	}
 
+	bin = g_new0 (GdaBinary, 1);
 	bin->data = (guchar*)retval;
 	bin->binary_length = len;
 
-	return TRUE;
+	return bin;
 }
 
 /**
@@ -2644,17 +2648,25 @@ gda_blob_to_string (GdaBlob *blob, guint maxlen)
 /**
  * gda_string_to_blob
  * @str: a string to convert
- * @blob: a non filled @GdaBlob structure
  *
  * Performs the reverse of gda_blob_to_string().
  *
- * Returns: TRUE if no error were found in @str, or FALSE otherwise
+ * Returns: a new #gdaBlob if no error were found in @str, or NULL otherwise
  */
-gboolean
-gda_string_to_blob (const gchar *str, GdaBlob *blob)
+GdaBlob *
+gda_string_to_blob (const gchar *str)
 {
-	gboolean retval;
-	retval = gda_string_to_binary (str, (GdaBinary*) blob);
-	blob->op = NULL;
-	return retval;
+	GdaBinary *bin;
+	bin = gda_string_to_binary (str);
+	if (bin) {
+		GdaBlob *blob;
+		blob = g_new0 (GdaBlob, 1);
+		((GdaBinary*) blob)->data = bin->data;
+		((GdaBinary*) blob)->binary_length = bin->binary_length;
+		blob->op = NULL;
+		g_free (bin);
+		return blob;
+	}
+	else
+		return NULL;
 }
